/*
	Gnash GTK2 XEmbed Demo
	Written (2016) by Nutchanon Wetchasit <Nutchanon.Wetchasit@gmail.com>
	
	Released to public domain under Creative Commons Zero 1.0
	(https://creativecommons.org/publicdomain/zero/1.0/)
	
	Note: This code is a simple example, it does not handle all kind of error.
	The error that this demo does not handle is:
		- Gnash terminated with error status.
		- User killed this demo (e.g. with `kill` command).
*/
#include <fcntl.h>
#include <signal.h>
#include <unistd.h>
#include <sys/wait.h>
#include <gtk/gtk.h>

/* Exit event handler */
void exiting() {
	gtk_main_quit();
}

/* Main program */
int main(int argc,char *argv[]) {
	GtkWidget *dialog;
	GtkWidget *window;
	GtkWidget *vbox;
	GtkWidget *label;
	GtkWidget *embed;
	GtkWidget *quit;
	GdkPixbuf *icon;
	GError *error;
	int x11_windowid;
	char x11_windowidstring[64];
	int errpipes[2];
	int pipeflag;
	char pipedata[1];
	int pid;
	
	gtk_init(&argc,&argv);
	
	/* Create window */
	window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window),"Gnash Inside");
	gtk_container_set_border_width(GTK_CONTAINER(window),3);
	g_signal_connect(window,"destroy",G_CALLBACK(exiting),NULL);
	
	/* Load icon */
	error=NULL;
	icon=gdk_pixbuf_new_from_file("icon.png",&error);
	if(icon!=NULL) {
		gtk_window_set_icon(GTK_WINDOW(window),icon);
	}
	
	/* Create label */
	label=gtk_label_new("Flash content displays below:");
	gtk_misc_set_alignment(GTK_MISC(label),0,0);
	
	/* Create XEmbed view */
	embed=gtk_socket_new();
	gtk_widget_set_size_request(GTK_WIDGET(embed),480,360);
	
	/* Create exit button */
	quit=gtk_button_new_with_mnemonic("E_xit");
	g_signal_connect(quit,"clicked",G_CALLBACK(exiting),NULL);
	
	/* Create layout */
	vbox=gtk_vbox_new(FALSE,3);
	gtk_container_add(GTK_CONTAINER(window),vbox);
	gtk_box_pack_start(GTK_BOX(vbox),label,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(vbox),embed,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(vbox),quit,FALSE,FALSE,0);
	
	/* Show window */
	gtk_widget_show_all(window);
	
	/* Fetch XEmbed window ID (window must be shown in order to do this) */
	x11_windowid=(int)gtk_socket_get_id(GTK_SOCKET(embed));
	sprintf(x11_windowidstring,"%d",x11_windowid);
	
	/* Run Gnash */
	if(pipe(errpipes)!=0 || (pipeflag=fcntl(errpipes[1],F_GETFD))==-1 || fcntl(errpipes[1],F_SETFD,pipeflag|FD_CLOEXEC)==-1) {
		dialog=gtk_message_dialog_new(
			GTK_WINDOW(window),
			GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_MESSAGE_ERROR,
			GTK_BUTTONS_OK,
			"Failed setting-up internal pipe."
		);
		gtk_window_set_title(GTK_WINDOW(dialog),"Error");
		gtk_dialog_run(GTK_DIALOG(dialog));
		return 1;
	}
	pid=fork();
	if(pid==0) {
		/* This is a child process: start Gnash */
		close(errpipes[0]);
		execlp("gtk-gnash","gtk-gnash","--xid",x11_windowidstring,"hovercolorbars.swf",NULL);
		
		/* If this code runs, failed starting Gnash, send error back to parent */
		pipedata[0]='!';
		write(errpipes[1],pipedata,1);
		close(errpipes[1]);
		return 127;
	} else {
		/* This is in parent process, if the child close pipe without any data, it runs successfully */
		close(errpipes[1]);
		
		if(read(errpipes[0],pipedata,1)>0) {
			/* There's data from child process, display error */
			while(read(errpipes[0],pipedata,1)>0);
			
			dialog=gtk_message_dialog_new(
				GTK_WINDOW(window),
				GTK_DIALOG_DESTROY_WITH_PARENT,
				GTK_MESSAGE_ERROR,
				GTK_BUTTONS_OK,
				"Failed to start Gnash."
			);
			gtk_window_set_title(GTK_WINDOW(dialog),"Error");
			gtk_dialog_run(GTK_DIALOG(dialog));
			gtk_widget_destroy(dialog);
		}
		close(errpipes[0]);
	}
	
	/* Run GUI loop until our program terminates */
	gtk_main();
	
	/* Terminate Gnash with SIGTERM */
	kill(pid,15);
	waitpid(pid,NULL,0);
	
	return 0;
}
